/*
 * $Id: DataStructManager.java, v0.0 2004/10/12$
 */
package datastruct;

import java.net.*;
import java.io.*;


import main.MainManager;
import viewer.ViewerConstants;

import request.PropagationRequest;

/*
import parser.*;
import parser.events.*;
import parser.events.metaevents.*;

import parser.subelements.State;
import parser.subelements.VList;
import parser.subelements.Variable;
import parser.subelements.ValueList;
*/

import datastruct.clpgui.Domains2D;
import datastruct.clpgui.Domains3D;
import datastruct.clpgui.SearchTree;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import java.util.zip.GZIPInputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.File;

import org.xml.sax.InputSource;


import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;

import java.util.Iterator;
import fr.inria.oadymppac.parsing.event.Event;
import fr.inria.oadymppac.parsing.event.EventListenerImpl;
import fr.inria.oadymppac.parsing.gentra4cp.event.BackToEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.BackToEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewVariableEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewVariableEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.SolutionEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.SolutionEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.ChoicePointEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.ChoicePointEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewConstraintEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewConstraintEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.FailureEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.FailureEventListener;

import fr.inria.oadymppac.parsing.gentra4cp.DefaultNewVariableEvent;


import fr.inria.oadymppac.parsing.info.ValueListInfo;
import fr.inria.oadymppac.parsing.info.ValueListProvider;
import fr.inria.oadymppac.parsing.info.VarDomainInfo;
import fr.inria.oadymppac.parsing.info.VarDomainProvider;
import fr.inria.oadymppac.parsing.info.VariableInfo;
import fr.inria.oadymppac.parsing.info.VariableListInfo;

import fr.inria.oadymppac.parsing.info.StateInfo;
import fr.inria.oadymppac.parsing.info.StateProvider;

import parser.PropagationSolver;

import parser.propagation.MetaPropagationListener;
import parser.propagation.MetaPropagationEvent;

/**
 * Christmas tree manager. This class builds the tree of propagation (reduce).
 * Each node represents a new constraint and its amount of propagation involved.
 *
 * @author Guillaume Arnaud
 * @version 1.0
 */
public class DataStructManager extends EventListenerImpl 
	implements NewVariableEventListener,BackToEventListener,ChoicePointEventListener,SolutionEventListener,   NewConstraintEventListener,FailureEventListener,MetaPropagationListener
{
	
	/** 
	 * Root of the propagation tree.
	 */
	private PropagationTree ctroot;   
    
    /** 
     * Current node of the propagation tree.
     */
    private PropagationTree ctCurrent;
    
	/** 
	 * Vector of nodes from propagation tree. Index = node number.
	 */
	private Vector propagationTreeVector;
	
    /** 
     * Vector of nodes from clpgui search tree. Index = node number.
     */
    private Vector clpguiSearchTreeVector;

	/** 
	 * Solver generating events.
	 */
	private PropagationSolver solver;
	
	/** 
	 * Current node number.
	 */
	private int node;

    /** 
     * Current node.
     */
    private String snode="";
    
    /** 
     * Domains2D.
     */
    private Domains2D domains2D;
    
    /** 
     * Domains3D.
     */
    private Domains3D domains3D;
    
    /** 
     * CLPGUI search tree.
     */
    private SearchTree searchTree;
    
    /** 
     * Manager containing this class.
     */
    private MainManager mainManager;
    
    /** 
     * Variable for pavot viewers (3D tree) existence.
     */
    private boolean pavotviewer=false;
    
    /** 
     * Variable for clpgui viewers existence.
     */
    private boolean clpguiviewer=false;


    private boolean hasJumpto=false;
    private int depth=0;
    private Hashtable nodeToDepth=new Hashtable();

	/** 
	 * Constructor. Notice: the order of listeners record is very important because
     * MetaPropagationListener contains SolverEvents. This listener must be the last
     * recorded.
	 * 
	 * @param solver0 handler generating trace events
	 * @param pr propagation request
	 * @param viewers vector of viewers
	 * @param mainManager  Manager containing this class
	 */
	public DataStructManager(PropagationSolver solver0, PropagationRequest pr,  Vector viewers, MainManager mainManager){
        super();
        this.mainManager=mainManager;
		solver=solver0;

        if(((Boolean)viewers.elementAt(ViewerConstants.PAVOT)).booleanValue()) {
            pavotviewer=true;
            propagationTreeVector=new Vector();
            propagationTreeVector.setSize(2);
	    	ctroot=new PropagationTree("root",0,pr.hasRequest());
	    	propagationTreeVector.setElementAt(ctroot,0);
	    	DataStructRequest ctr=createDataStructRequest(null,null);
	    	ctroot.setDataStructRequest(ctr);
	    	ctCurrent=ctroot;
        }

        if(((Boolean)viewers.elementAt(ViewerConstants.CLPGUI)).booleanValue()){
            clpguiviewer=true;
            clpguiSearchTreeVector=new Vector();
    		clpguiSearchTreeVector.setSize(2);
            domains2D=new Domains2D();
            domains3D=new Domains3D();
            domains3D.setRoot(new SearchTree("root"));
            searchTree=domains3D.getRoot();
            domains3D.setCurrent(searchTree);
            domains3D.addExecutedGoals(searchTree);
            domains3D.addDomains2D(new Domains2D());
            domains3D.incTime();
            clpguiSearchTreeVector.setElementAt(domains3D.getRoot(),0);
        }

        try{
            solver.addDefaultListener(this);
      /*      if(clpguiviewer)    solver.addListener(solver.addElement("new-variable",DefaultNewVariableEvent.class),this);
            if(clpguiviewer||pavotviewer){
                solver.addChoicePointListener(this);
                solver.addSolutionListener(this);
                solver.addBackToListener(this);
                solver.addFailureListener(this);
            }*/
           if(pavotviewer) solver.addMetaPropagationListener(this, pr);
        }
        catch(Exception exception){System.out.println("Fail to add listener in DataStructManager");}
	}
	protected void showValueList(Event ev) {
            ValueListProvider valueList = (ValueListProvider) ev;
            showValueList(valueList.getValueList());
         }
        
    protected void showValueList(ValueListInfo dom) {
            Iterator it=(dom == null ? null : dom.iterator());
            System.out.print("DOMAIN:< ");
            while (it !=null && it.hasNext())
                System.out.print((it.next()).toString()+" ");
            System.out.println(" >");
        }
    public void eventProcess(NewVariableEvent e){
        if(clpguiviewer){
            String vident=e.getIdentifier();
            String vname=e.getName();
            if(vname==NewVariableEvent.NAME_NONE) return; //domains2D.addVariable(vident);
            else domains2D.addVariable(vname,vident);
            

           VarDomainProvider varDomainProvider= (VarDomainProvider) e;
           VarDomainInfo varDomainInfo=varDomainProvider.getVarDomain();
           ValueListInfo valueList=varDomainInfo.getValueList();
            
            if(valueList!=null) {
                Vector domain= new Vector(valueList.getInfo());           
                if(domain!=null){  
                    domains2D.setDomain(vident, domain);
                }
        /*        System.out.println("[DataStructManager] new var 1");
                mainManager.refresh();
                System.out.println("[DataStructManager] new var 2");*/
            }
        }
    }

	/** 
	 * Adds a new child at the current node.
	 * 
	 * @param e ChoicePointEvent
	 */
	public void eventProcess(ChoicePointEvent e){
        snode=e.getNode();
        try{
    		node=Integer.parseInt(snode);
            this.linkNodeToDepth(snode,new Integer(++depth));
        }
        catch(Exception exc){
            System.out.print("Warning: choice-point event without node number. Try an other way...");
            try{
                int depthEv=e.getDepth();
                if(depthEv<=depth){
                    if(pavotviewer)	ctCurrent=(PropagationTree)propagationTreeVector.elementAt(depthEv);
                    if(clpguiviewer) {
                        searchTree=(SearchTree)clpguiSearchTreeVector.elementAt(depthEv);
                        domains3D.backto(searchTree);
                    }                    
                }
                depth=depthEv;
                System.out.println("[Successed]");
            }
            catch(Exception exc2){
                System.out.println("[Failed]");
            }
        }

        if(clpguiviewer){
            StateProvider state=(StateProvider) e;
            StateInfo stateInfo=state.getState();
           
            String property="";
            if(e.getContext()!=null && e.getContext().startsWith("dummy"))
                property="show";
            this.addCLPGUIChild("","n="+node);
            
            if(stateInfo!=null) {
                //Variable 2D
                this.setCLPGUIDomains2D(stateInfo);
                //Variable 3D
                this.setCLPGUIDomains3D();
            }
        }
        ///////////////
        if(pavotviewer){
            this.addPavotChild("choice-point");
        }
        
        if(pavotviewer||clpguiviewer) mainManager.refresh();
	}
    
	/** 
	 * Adds a solution node.
	 * 
	 * @param e SolutionEvent
	 */
	public void eventProcess(SolutionEvent e) {
        snode=e.getNode();
        try{
    		node=Integer.parseInt(snode);
            this.linkNodeToDepth(snode,new Integer(++depth));
        }
        catch(Exception exc){
            System.out.println("Warning: solution event without node number. Try an other way...");
            try{
                depth++;
                System.out.println("[Successed]");
            }
            catch(Exception exc2){
                System.out.println("[Failed]");
            }
        }        
        
        if(clpguiviewer){
            StateProvider state=(StateProvider) e;
            StateInfo stateInfo=state.getState();
            
            this.addCLPGUIChild("success","solution n="+node);
            
            if(stateInfo!=null) {
                //Variable 2D
                this.setCLPGUIDomains2D(stateInfo);
                //Variable 3D
                this.setCLPGUIDomains3D();
            }        
        }
        
        ///////////////
        if(pavotviewer){
            this.addPavotChild("solution");
        }
        
        if(pavotviewer||clpguiviewer) mainManager.refresh();
	}

    public void eventProcess(FailureEvent e){
        snode=e.getNode();
        try{
    		node=Integer.parseInt(snode);
            this.linkNodeToDepth(snode,new Integer(++depth));
        }
        catch(Exception exc){
            System.out.println("Warning: failure event without node number. Try an other way...");
            try{
                depth++;
                System.out.println("[Successed]");
            }
            catch(Exception exc2){
                System.out.println("[Failed]");
            }
            
        }        
        
        if(clpguiviewer){
            StateProvider state=(StateProvider) e;
            StateInfo stateInfo=state.getState();
            
            this.addCLPGUIChild("failure","failure n="+node);
            
            if(stateInfo!=null) {
                //Variable 2D
                this.setCLPGUIDomains2D(stateInfo);
                //Variable 3D
                this.setCLPGUIDomains3D();
            }
        }
        
        ///////////////
        if(pavotviewer){
            this.addPavotChild("failure");
        }
        
    }
    
	/** 
	 * Returns to the relevant node.
	 * 
	 * @param e BackToEvent
	 */
	public void eventProcess(BackToEvent e){
       try{
        int newNode;
        if(hasJumpto) newNode=Integer.parseInt(e.getNode());
        else{
            newNode=this.getDepthFromNode(e.getNode());
            depth=newNode;
        }
        
        if(pavotviewer)	ctCurrent=(PropagationTree)propagationTreeVector.elementAt(newNode);
        if(clpguiviewer) {
            searchTree=(SearchTree)clpguiSearchTreeVector.elementAt(newNode);
            domains3D.backto(searchTree);
        }
       }
       catch(Exception exc){
           System.out.println("Warning: back-to event without node number. Try an other way...");
           try{
               depth=e.getDepth();
               System.out.println("[Successed]");
               if(pavotviewer)	ctCurrent=(PropagationTree)propagationTreeVector.elementAt(depth);
               if(clpguiviewer) {
                   searchTree=(SearchTree)clpguiSearchTreeVector.elementAt(depth);
                   domains3D.backto(searchTree);
               }                 
           }
           catch(Exception exc2){System.out.println("[Failed]");}
       }
	}

    public void eventProcess(Event e) {}
    /** 
     * 
     * 
     * @param e 
     */
    public void notifyMetaPropagationListener(MetaPropagationEvent e) {
        if(pavotviewer){
    		DataStructRequest ctr=createDataStructRequest(e.getFilteredConstraint(),e.getTotalEvents());
	    	ctCurrent.setNbTotalPropagation(e.getNbTotalPropagation());
	    	ctCurrent.setDataStructRequest(ctr);
	    	ctCurrent.setExternalRep(e.getConstraintNames());
        }
    }
    
	/** 
	 * Creates a DataStructRequest which contains information about propagation.
	 * 
	 * @param fc 
	 * @param te 
	 * @return 
	 */
	private DataStructRequest createDataStructRequest(Hashtable fc,Hashtable te){
		DataStructRequest ctr=new DataStructRequest();		
		Enumeration e;
		
		if(fc!=null){
			e=fc.keys();
			while(e.hasMoreElements()){				
				String s=(String)(e.nextElement());
				ctr.addRequest(s,((Integer)te.get(s)).intValue(),(boolean)((Hashtable)fc.get(s)!=null),(Hashtable)fc.get(s));
			}
		}
		return ctr;		
	}

	/**
	 * Gets propagation tree.
	 * @return ctroot PropagationTree
	 */
	public PropagationTree getPropagationTree(){ return ctroot; }
	public SearchTree getSearchTree(){ return (SearchTree)clpguiSearchTreeVector.firstElement(); }

    public Domains2D getDomains2D() { return domains2D;}
    public Domains3D getDomains3D() { return domains3D;}

	public void  eventProcess(NewConstraintEvent e){
        //System.out.println("[DataStructManager] NewConstraint chrono="+e.getChrono());
	}

    private void setCLPGUIDomains2D(StateInfo stateInfo){
        VariableInfo info;
        VariableListInfo variables=stateInfo.getVariableList();
        Iterator it=(variables == null ? null : variables.iterator());
        while (it !=null && it.hasNext()) {
            info = (VariableInfo)it.next();
            VarDomainInfo varDomain = info.getVarDomain();
            ValueListInfo valueList = varDomain.getValueList();
            if(valueList!=null) {
                if(!valueList.isEmpty()) {
                    domains2D.setDomain(info.getIdentifier(),(Vector)valueList.getInfo());}
                else { 
                    try{
                        domains2D.setDomain(info.getIdentifier(),varDomain.getMin(),varDomain.getMax());}
                    catch(Exception exce){
                        try{
                            domains2D.setDomain(info.getIdentifier(),varDomain.getSize());}
                        catch(Exception exce2){System.out.println("Error Domain: not enough information");}
                    }
                }
            }
        }         
   
    }
    private void setCLPGUIDomains3D(){
        domains3D.setCurrent(searchTree);
        domains3D.addExecutedGoals(searchTree);
        domains3D.addDomains2D(domains2D);
        domains3D.incTime();
    }

    private void addCLPGUIChild(String property, String label){
        SearchTree newSearchTree=new SearchTree();
        newSearchTree.property+=property;
        newSearchTree.label+=label;
        searchTree.addChild(newSearchTree);
        searchTree=newSearchTree;

        int index;
        if(hasJumpto) index=node;
        else index=depth;

        if(clpguiSearchTreeVector.size()<=index) clpguiSearchTreeVector.setSize(index+2);
        clpguiSearchTreeVector.setElementAt(newSearchTree,index);
    }

    private void addPavotChild(String property){
            int index;
            if(hasJumpto) index=node;
            else index=depth;

            PropagationTree ctnew=new PropagationTree(ctCurrent.hasPropagation());
            ctnew.setLabel(property+", n="+index);
            ctnew.setNodeNumber(index);
            ctCurrent.addChild(ctnew);
            ctCurrent=ctnew;
            
            if(propagationTreeVector.size()<=index) propagationTreeVector.setSize(index+2);
            propagationTreeVector.setElementAt(ctnew,index);

    }

    private void linkNodeToDepth(String n, Integer d){
        nodeToDepth.put(n,d);
    }

    private int getDepthFromNode(String n) {
        return ((Integer)nodeToDepth.get(n)).intValue();
    }


}
