/*
 * $Id: DataStructRequest.java, v0.0 2003/10/02$
 */

package datastruct;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * Data structures for Christmas Tree request.
 * @author Guillaume Arnaud
 * @version 0.0
 */

public class DataStructRequest{
	Vector typeList;
	private Hashtable requests;
	int uselessWakeup=0;

	/**
	 * Constructor.
	 */
	public DataStructRequest(){requests=new Hashtable();}

	/**
	 * Adds a request.
	 */
	public void addRequest(String t, boolean f, Vector c){
		DataStructOneRequest newRequest=new DataStructOneRequest(t,f);
		
		if(requests!=null && requests.containsKey(t)) {
			System.out.println("Error DataStructRequest.addRequest(String,boolean,Vector) : Existing type of request");
			return;
		}
		if(requests==null) {
			requests=new Hashtable();
		typeList=new Vector();
		}
		if(f&&c!=null){
			Enumeration e=c.elements();
			while(e.hasMoreElements()){
				newRequest.addConstraintFilter((String)e.nextElement(),0);}

			requests.put(t,newRequest);
			typeList.addElement(t);
		}
		else requests.put(t,newRequest);
	}

	public void addRequest(String t, String c){
		if(requests!=null && requests.containsKey(t)) {
			DataStructOneRequest ctor=(DataStructOneRequest)requests.get(t);
			ctor.addConstraintFilter(c,0);
			return;
		}
	}
	
	public void addRequest(String t, int nb, boolean b, Hashtable h){
		DataStructOneRequest ctor;
		
		if(requests==null) requests=new Hashtable();
		if(h!=null) b=true;
		if(requests.containsKey(t)){
			ctor=(DataStructOneRequest)requests.get(t);
			ctor.init(t,nb,b,h);
		}
		else{
			ctor=new DataStructOneRequest();
			ctor.init(t,nb,b,h);
			requests.put(t,ctor);
		}
	}
	
	/**
	 * Gets requests.
	 */
	public Hashtable getRequests() {return requests;}
	
	/**
	 * Has requests req?
	 */
	public boolean hasRequest(String req) {
		return requests.containsKey(req);
	}
	
	/**
	 * Sets the useless awakes.
	 */
	public void setUselessWakeup(int n) { uselessWakeup=n;}
	
	/**
	 * Gets the useless awakes.
	 */
	public int getUselessWakeup() { return uselessWakeup;}
		
	/**
	 * Sets constraint filter value.
	 */
	public void setConstraintFilterValue(int v, String t, String c){
		if(!requests.containsKey(t)) { return ;}
		((DataStructOneRequest)requests.get(t)).setConstraintFilterValue(v,c);
		return;
	}
	public void setConstraintFilterValue(Hashtable a, String t){
		if(!requests.containsKey(t)) { return ;}
		((DataStructOneRequest)requests.get(t)).setConstraintFilterValue(a);
		return;
	}
	/** 
	 * Gets typeList
	 */
	public Vector getTypeList(){return typeList;}

	/**
	 * Gets constraint filter.
	 */
	public Hashtable getConstraintFilter(String t){
		if(!requests.containsKey(t)) { return null;}
		return ((DataStructOneRequest)requests.get(t)).getConstraintFilter();
	}

	/**
	 * Gets constraint filter value.
	 */
	public int getConstraintFilterValue(String t, String c){
		if(!requests.containsKey(t)) { return 0;}
		return ((DataStructOneRequest)requests.get(t)).getConstraintFilterValue(c);
	}
	
	/**
	 * Is a filter request?.
	 */
	public boolean isFilter(String t){
		if(requests==null || !requests.containsKey(t)) return false;
		else return ((DataStructOneRequest)requests.get(t)).isFilter();
	}
	
	
	/**
	 * Sets the total of events for this request.
	 */
	public void setTotalEvents(int v, String t){ 
		if(!requests.containsKey(t)) { return ;}
		((DataStructOneRequest)requests.get(t)).setTotalEvents(v);
		return;
	}

	/**
	 * Gets the total of events for this request.
	 */
	public int getTotalEvents(String t){
		if(!requests.containsKey(t)) { 
			return 0;}
		return ((DataStructOneRequest)requests.get(t)).getTotalEvents();
	}
	     
	/**
	 * Copies.
	 */
	public DataStructRequest copy(){
		DataStructRequest ctr=new DataStructRequest();
		Enumeration e;
		
		if(requests!=null) {
			e=requests.elements();
			while(e.hasMoreElements()){
				DataStructOneRequest ctor=((DataStructOneRequest)e.nextElement()).copy();
				ctr.requests.put(ctor.type,ctor);			
			}
		}
		return ctr;
	}
	/**
	 * Data structure for Christmas Tree requests.
	 */
	private class DataStructOneRequest extends Object {
		boolean filter=false; // are there filters for differents constraints?
		String type=""; // reduce, activate, wakeup...
		Hashtable constraintFilter=null; // number of instance of each constraints.
		Vector constraintFilterVec=null;

		int totalEvents;

		/**
		 * Constructor.
		 */
		public DataStructOneRequest(){
		}

		public DataStructOneRequest(String t, String c) {
			filter=true;
			type=t;
			constraintFilter=new Hashtable();
			constraintFilterVec=new Vector();
			constraintFilter.put(c,new Integer(0));
		}

		public DataStructOneRequest(String t, boolean f) {
			filter=f;
			type=t;
			if(f) constraintFilter=new Hashtable();
		}

		/**
		 * Copies.
		 */
		public DataStructOneRequest copy() {
			DataStructOneRequest ctor=new DataStructOneRequest();
			ctor.init(this.type,this.totalEvents,this.filter,this.constraintFilter);
			return ctor;
		}

		/**
		 * Adds a constraint filter.
		 */
		public void addConstraintFilter(String c, int nb) {
			Integer i=new Integer(nb);
			filter=true;
			if(constraintFilter==null||constraintFilterVec==null) {
				constraintFilter=new Hashtable();
				constraintFilterVec=new Vector();
			}
			constraintFilter.put(c,new Integer(nb));
			constraintFilterVec.addElement(c);
		}

		/**
		 * Initializes the request.
		 */
		public void init(String s,int nb, boolean b, Hashtable h) {
			type=s;
			constraintFilter=(Hashtable)h.clone();
			totalEvents=nb;
			filter=b;
		}
		
		/**
		 * Sets value for constraint filter.
		 */
		public void setConstraintFilterValue(int v, String c){
			if(!constraintFilter.containsKey(c)) return;
			else {
				constraintFilter.put(c,new Integer(v));
			}
		}
		public void setConstraintFilterValue(Hashtable a){
			constraintFilter=a;
		}

		/**
		 * Gets constraint filter.
		 */
		public Hashtable getConstraintFilter(){
			return constraintFilter;
		}	
		
		/**
		 * Gets value for constraint filter.
		 */
		public int getConstraintFilterValue(String c){
			if(!constraintFilter.containsKey(c)) {
				return 0;
			}
			else {
				return ((Integer)constraintFilter.get(c)).intValue();
			}
		}
		/**
		 * Sets the total of events for this request.
		 */
		public void setTotalEvents(int t){ totalEvents=t;}
		
		/**
		 * Gets the total of events for this request.
		 */
		public int getTotalEvents() { return totalEvents;}
	
		/**
		 * Is filter ?
		 */
		public boolean isFilter() {return filter;}
		
	}		
	
}

