/* file name  : datastruct/clpgui/Domains2D.java
 * authors    : Guillaume Arnaud
 * created    : jeu 01 avr 2004 16:31:12 CEST
 * copyright  : 
 *
 * modifications:
 *
 */

package datastruct.clpgui;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import fr.inria.oadymppac.parsing.info.ValuesInfo;
import fr.inria.oadymppac.parsing.info.RangeInfo;
/** 
 * Data structure for 2D domains. Records the names of variable (default name if it doesn't
 * exist), the identifiers (required), domains of each variable, minimum and maximum of 
 * all domains.
 * 
 * @author Guillaume Arnaud
 * @version 1.0
 * @see CLPGUIDomains2D
 */
public class Domains2D implements Cloneable{
    
    /** 
     * Minimum value of domains. Initialized to {@value}.
     */
    private int vMin=10000;    
    
    /** 
     * Maximum value of domains. Initialized to {@value}.
     */
    private int vMax=-10000;

    /** 
     * External representation of variables.
     */
    private Vector names;

    /** 
     * Hashtable of domains.
     */
    private Hashtable domains;

    /** 
     * Vector of identifier.
     */
    private Vector idents;
    /** 
     * Number of variables.
     */
    private int nbvar=0;

    /** 
     * 
     */
    private Hashtable domainsSize;

    /** 
     * Constant. Domain is represented by its size.
     */
    public static final int SIZEDOMAIN=1;
    
    /** 
     * Constant. Domain is represented by an interval.
     */
    public static final int INTERVALDOMAIN=2;
    
    /** 
     * Constant. Domain is represented by its values.
     */
    public static final int VALUEDOMAIN=3;
    
    /** 
     * Information about representation of domains.
     */
    private int infoDomain=VALUEDOMAIN;
    /** 
     * Constructor.
     */
    public Domains2D(){
        domains=new Hashtable();
        names=new Vector();
        idents=new Vector();
        domainsSize=new Hashtable();
    }

    public int getInfoDomain(){return infoDomain;}
    /** 
     * Gets number of variables.
     * 
     * @return number of variables.
     */
    public int getNbVar() {return nbvar;}

    /** 
     * Sets the number of variables.
     * 
     * @param nbvar the new number of variables.
     */
    public void setNbVar(int nbvar) {this.nbvar=nbvar;}

    /** 
     * Gets minimum value.
     * 
     * @return vMin
     */
    public int getVMin(){return vMin;}
    
    public void setVMin(int min){vMin=min;}
    
    /** 
     * Gets maximum value.
     * 
     * @return vMax
     */
    public int getVMax(){return vMax;}
    public void setVMax(int max){vMax=max;}
    
    public void resize(Vector domain) {
        int newMin=vMin, newMax=vMax;
        Enumeration enu=domain.elements();
        while(enu.hasMoreElements()){
            Object o=enu.nextElement();
            try { 
                newMin= this.getMin((ValuesInfo) o);
                newMax= this.getMax((ValuesInfo) o);
            }
            catch(Exception ex1) {
                try {
                    newMin= this.getMin((RangeInfo) o);
                    newMax= this.getMax((RangeInfo) o);
                }
                catch(Exception ex2) {//System.out.println("Error in domain");
                }
            }
            if(newMin<vMin) vMin=newMin;
            if(newMax>vMax) vMax=newMax;
        }
    }

    public int getMin(ValuesInfo vi){
        if(vi.getValues().length==0) return 10000;
        int[] viTab=vi.getValues();
        int min=viTab[0],i;
        for(i=1;i<viTab.length;i++) { if(viTab[i]<min) min=viTab[i];}
        return min;        
    }

    public int getMax(ValuesInfo vi){
        if(vi.getValues().length==0) return -10000;
        int[] viTab=vi.getValues();
        int max=viTab[0],i;
        for(i=1;i<viTab.length;i++) { if(viTab[i]>max) max=viTab[i];}
        return max;        
    }
    public int getMin(RangeInfo ri){
        return ri.getFrom();        
    }

    public int getMax(RangeInfo ri){
        return ri.getTo();        
    }
    
    /** 
     * Adds a variable with its identifier, then increments the
     * number of variable.
     * 
     * @param ident identifier of the variable.
     */
    public void addVariable(String ident) {
        names.addElement(""+ident);
        idents.addElement(ident);
        domainsSize.put(ident,new Integer(0));
        nbvar++;
    }    
    
    /** 
     * Adds a variable with its name and its identifier, then increments the
     * number of variable.
     * 
     * @param var the name of the new variable.
     * @param ident identifier of the variable.
     */
    public void addVariable(String var, String ident) {
        names.addElement(var);
        idents.addElement(ident);
        domainsSize.put(ident,new Integer(0));
        nbvar++;
    }    

    /** 
     * Sets a domain dom to the variable with identifier ident, changes minimum
     * and maximum of domain if necessary.
     * 
     * @param ident the String of the identifier
     * @param dom the Vector of values
     * @param min the int of the minimum value of dom
     * @param max the int of the maximum value of dom
     */
    public void setDomain(String ident, Vector dom, int min, int max){       
        domainsSize.put(ident,new Integer(this.domainsResize(dom)));
        domains.put(ident,dom);
        if(min<vMin) vMin=min;
        if(max>vMax) vMax=max;
    }
    /** 
     * Sets a domain dom to the variable with identifier ident.
     * 
     * @param ident the String of the identifier
     * @param dom the Vector of values
     */
    public void setDomain(String ident, Vector dom){       
        domainsSize.put(ident,new Integer(this.domainsResize(dom)));
        domains.put(ident,dom);
        infoDomain=VALUEDOMAIN;
        this.resize(dom);
    }

    public void setDomain(String ident, int min, int max){
        Vector dom=new Vector();
        RangeInfo ri=new RangeInfo();

        ri.setFrom(min);
        ri.setTo(max);
        dom.addElement(ri);
        domainsSize.put(ident,new Integer(this.domainsResize(dom)));
        domains.put(ident,dom);
        infoDomain=INTERVALDOMAIN;
        this.resize(dom);
    }

     public void setDomain(String ident, int size){
        Vector dom=new Vector();
        RangeInfo ri=new RangeInfo();

        ri.setFrom(1);
        ri.setTo(size);
        dom.addElement(ri);
        domainsSize.put(ident,new Integer(size));
        domains.put(ident,dom);
        infoDomain=SIZEDOMAIN;
        this.resize(dom);
    }
   
    public int getSize(int var) {
        int ret=0;
        String ident=this.getIdent(var);
        if(ident==null) return ret;
        Vector dom=(Vector)domains.get(ident);
        if(dom!=null) ret=domainsResize(dom);
        return ret;
    }

    public int domainsResize(Vector dom){
        Enumeration enu=dom.elements();
        int newSize=0;
        while(enu.hasMoreElements()){
            Object o=enu.nextElement();
            try { 
                newSize=newSize+ this.getSize((ValuesInfo) o);
            }
            catch(Exception ex1) {
                try {
                    newSize=newSize+ this.getSize((RangeInfo) o);
                }
                catch(Exception ex2) {//System.out.println("Error in domain");}
            }
            }
        }
        return newSize;
    }

    private int getSize(ValuesInfo vi){
        return vi.getValues().length;
    }

    private int getSize(RangeInfo ri){
        return ri.getTo()-ri.getFrom()+1;
    }
    /** 
     * Gets name variable at index var.
     * 
     * @param var int index of the variable
     * @return String of the variable name.
     */
    public String getName(int var){
        if(var<names.size()) return (String)names.elementAt(var);
        else return new String("");
    }

    public void setNames(Vector names){
        this.names=names;
    }
    public Vector getNames(){
        return this.names;
    }

    /** 
     * Gets idents at index var.
     * 
     * @param var int identifier of the variable.
     * @return String of the identifier name.
     */
    public String getIdent(int var) {
        if(var<idents.size()) return (String)idents.elementAt(var);
        else return new String("");
    }

    public void setIdents(Vector idents){
        this.idents=idents;
    }
    public void setDomainsSize(Hashtable domainsSize){
        this.domainsSize=domainsSize;
    }
    public void setDomains(Hashtable domains){
        this.domains=domains;
    }
    
    /** 
     * Gets domains.
     * 
     * @return Hashtable domains.
     */
    public Hashtable getDomains() {return domains;}
    public Object clone() throws CloneNotSupportedException {
        Domains2D clone=new Domains2D();
        clone.setNames((Vector)this.getNames().clone());
        clone.setVMin(this.vMin);
        clone.setVMax(this.vMax);
        clone.setNbVar(this.nbvar);
        clone.setIdents((Vector)this.idents.clone());
        clone.setDomainsSize((Hashtable)this.domainsSize.clone());
        clone.setDomains((Hashtable)this.domains.clone());
        return clone;}
        
    public String domainToString(int v){
        String ident=this.getIdent(v);
        if(ident==null || ident=="") return "";
        else{
            Vector dom=(Vector) domains.get(ident);
            if(dom==null) return "";
            else{
                String ret="";
                Enumeration enu=dom.elements();
                while(enu.hasMoreElements()){
                Object o=enu.nextElement();
                try { 
                    ret+= ((ValuesInfo) o).toString();
                }
                catch(Exception ex1) {
                    try {
                        ret+= ((RangeInfo) o).toString();
                    }
                    catch(Exception ex2) {//System.out.println("Error in domain");
                    }
                }
                }
                   
                return ret;
            }
        }
    }

    public int[] domainToVector(int v){
        String ident=this.getIdent(v);
        if(ident==null || ident=="") return null;
        else{
            Vector dom=(Vector) domains.get(ident);
            if(dom==null) return null;
            else{
                int[] ret=new int[this.getSize(v)];
                int incr=0;
                Enumeration enu=dom.elements();
                while(enu.hasMoreElements()){
                Object o=enu.nextElement();
                try { 
                    int[] values=((ValuesInfo) o).getValues();
                    for(int i=0; i<values.length;i++){                      
                        ret[incr++]=values[i];
                    }
                }
                catch(Exception ex1) {
                    try {
                      RangeInfo ri=(RangeInfo) o;
                      for(int i=ri.getFrom(); i<=ri.getTo();i++){
                         ret[incr++]=i;
                      }
                    }
                    catch(Exception ex2) {//System.out.println("Error in domain");
                    }
                }
                }
                   
                return ret;
            }
        }
        
    }
    
        
} // END: Domains2D
