/* file name  : viewer/CLPGUICanvas3D.java
 * authors    : Guillaume Arnaud
 * created    : ven 09 avr 2004 14:55:08 CEST
 * copyright  : 
 *
 * modifications:
 *
 */

package viewer;

import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import java.awt.Canvas;
import java.awt.CheckboxMenuItem;
import java.awt.MenuBar;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Font;
import java.awt.Color;

/** 
 * 
 * 
 * @author Guillaume Arnaud
 * @version 1.0
 */
public class CLPGUICanvas3D extends Canvas implements MouseListener, MouseMotionListener, ActionListener, ItemListener{
	MenuBar menuBar;
	Menu display;
	double zoomX, zoomY;
	CheckboxMenuItem rotate, translate, zoom;
	MenuItem center;

	int width=500;
	int height=450;
	Graphics gMem;    // double buffering 
	Image memory;

	double oX;
	double oY;
	int lastX,lastY,firstX,firstY;
	
	int[] x=new int[6]; // temporary points
	int[] y=new int[6];

	double aY,aZ;
	double sinRZ, cosRZ, sinRY, cosRY;
	
	static final double pi6=Math.PI/6;
	static final double pi4=Math.PI/4;
	static final double pi2=Math.PI/2;

    boolean refresh=true;

	public CLPGUICanvas3D() {
	    setSize(width,height);
	    memory=null;
	    addMouseListener(this);
	    addMouseMotionListener(this);

	    menuBar=new MenuBar();
	    display=new Menu("Geometry");
	    menuBar.add(display);

	    rotate=new CheckboxMenuItem("Rotate",true);
	    translate=new CheckboxMenuItem("Translate",false);
	    zoom=new CheckboxMenuItem("Zoom",false);
	    center=new MenuItem("Center");
	    center.addActionListener(this);
	    rotate.addItemListener(this);
	    translate.addItemListener(this);
	    zoom.addItemListener(this);
	    display.add(center);
	    display.add(rotate);
	    display.add(translate);
	    display.add(zoom);

	    zoomX=1; // can't use center() because overiding...
	    zoomY=1;
	    oX=0;
	    oY=0;
	    aY=0;
	    aZ=0;
	    initRotate();
        this.setVisible(true);
	}

	void center() {
	    zoomX=1;
	    zoomY=1;
	    oX=0;
	    oY=0;
	    aY=0;
	    aZ=0;
	    initRotate();
	    refresh=true;
	    repaint();
	}

	void resized(int w, int h) {
	    width=w;
	    height=h;
	    setSize(width,height);
	    memory=null;
	    refresh=true;
	}

	// Rotation

	void initRotate() {
	    sinRZ=Math.sin(aZ);
	    cosRZ=Math.cos(aZ);
	    sinRY=Math.sin(aY);
	    cosRY=Math.cos(aY);
	}

	// Projection from xyz into ij
	int i(double x, double y, double z) {
	    return (int)(oX+zoomX*(y*cosRZ+x*sinRZ));
	}
	
	int j(double x, double y, double z) {
	    return (int)(oY+zoomY*(-z*cosRY+(x*cosRZ-y*sinRZ)*sinRY));
	}

	// Inverse projection from ij into xy when z=0
	double x(int ii, int jj) {
	    double i=(ii-oX)/zoomX;
	    double j=(jj-oY)/zoomY;
	    return (i-y(ii,jj)*cosRZ)/sinRZ;
	}
	
	double y(int ii, int jj) {
	    double i=(ii-oX)/zoomX;
	    double j=(jj-oY)/zoomY;
	    return i*cosRZ-j*sinRZ/sinRY;
	}

	// Inverse projection from ij into xy knowing z
	double x(int ii, int jj, double z) {
	    double i=(ii-oX)/zoomX;
	    double j=(jj-oY)/zoomY;
	    return (i-y(ii,jj,z)*cosRZ)/sinRZ;
	}
	
	double y(int ii, int jj, double z) {
	    double i=(ii-oX)/zoomX;
	    double j=(jj-oY)/zoomY;
	    return i*cosRZ-(j+z*cosRY)*sinRZ/sinRY;
	}

	// Menus
	public void actionPerformed(ActionEvent e) {
	    if (e.getSource()==center) {
		center();
	    }
	}

	public void itemStateChanged(ItemEvent e) {
	    String s=e.getItem().toString();
	    
	    if (s.equals("Rotate") && rotate.getState()) {
		translate.setState(false);
		zoom.setState(false);
	    }
	    else if (s.equals("Translate") && translate.getState()) {
		rotate.setState(false);
		zoom.setState(false);
	    }
	    else if (s.equals("Zoom") && zoom.getState()){
		translate.setState(false);
		rotate.setState(false);		
		zoom.setState(true);		
	    }
	    //else return;

	    refresh=true;
	    repaint();
	}
    
	// Mouse events
	public synchronized void mouseClicked(MouseEvent e)
	{
	    if (rotate.getState()) {
		rotate.setState(false);
		translate.setState(true);
		zoom.setState(false);
	    }
	    else if (translate.getState()) {
		rotate.setState(false);
		translate.setState(false);
		zoom.setState(true);
	    }
	    else if (zoom.getState()) {
		rotate.setState(true);
		translate.setState(false);
		zoom.setState(false);
	    }
	}

	public synchronized void mousePressed(MouseEvent e)
	{
	    lastX=e.getX();
	    lastY=e.getY();
	    firstX=lastX;
	    firstY=lastY;
	}

	public void mouseReleased(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}

	public synchronized void mouseDragged(MouseEvent e)
	{
	    if (rotate.getState()) {
		double raZ=aZ;
		double raY=aY;
		aZ=aZ+(e.getX()-lastX)/300.0;
		if (aZ<-pi2 || aZ>0) aZ=raZ;
		aY=aY+(e.getY()-lastY)/300.0;
		if (aY<0 || aY>pi2) aY=raY;
		sinRZ=Math.sin(aZ);
		cosRZ=Math.cos(aZ);
		sinRY=Math.sin(aY);
		cosRY=Math.cos(aY);
		lastX=e.getX();
		lastY=e.getY();
	    }
	    else if (translate.getState()) {
		oX=oX+e.getX()-lastX;
		oY=oY+e.getY()-lastY;
		lastX=e.getX();
		lastY=e.getY();
	    }
	    else {
		double dZoomX=(e.getX()-lastX)*0.01;
		double dZoomY=(e.getY()-lastY)*0.01;
		// zoom with a translation that preserves the coordinate of the first pressed point
		// a negative zoom reverses the figure
  		if (zoomX+dZoomX>0.01 || zoomX+dZoomX<-0.01) {
		    oX=firstX-(firstX-oX)*(1+dZoomX/zoomX);
		    zoomX=zoomX+dZoomX;
		    lastX=e.getX();
		}
  		if (zoomY+dZoomY>0.02 || zoomY+dZoomY<-0.02) {
		    oY=firstY-(firstY-oY)*(1+dZoomY/zoomY);
		    zoomY=zoomY+dZoomY;
		    lastY=e.getY();
		}
	    }
	    refresh=true;
	    paint(getGraphics());
	}

	public void mouseMoved(MouseEvent e) {}

	public synchronized void paint(Graphics g)     
	{
	    if (memory==null) {
		memory = createImage(width,height);
		gMem = memory.getGraphics();
		gMem.setFont(new Font("monospaced",Font.PLAIN,12));
		refresh=true;
	    }

	    if (refresh) {
		gMem.setColor(Color.white);
		gMem.fillRect(0, 0, width, height);
	    }
	}
    }

