package viewer;

import request.PropagationRequest;

import datastruct.*; 
import gui.UISolverSelection;

import java.util.zip.GZIPInputStream;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.Math;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.*;
import javax.swing.event.*;

import parser.PropagationHandler;

public class CTViewer extends Object implements PavotViewer {
    int type=PavotViewer.PROPAGATIONTREE;
	// Static variables
	static final int FRAMEWIDTH=600, FRAMEHEIGHT=600;
	static final int CANVASWIDTH=600, CANVASHEIGHT=600;
	static final int SCROLLHORSMOOTH=20,SCROLLHORVAL=CANVASWIDTH*SCROLLHORSMOOTH/2, SCROLLHORMIN=-10*CANVASWIDTH*SCROLLHORSMOOTH,SCROLLHORMAX=10*CANVASWIDTH*SCROLLHORSMOOTH;
	static final int SCROLLVERSMOOTH=20,SCROLLVERVAL=-20*SCROLLVERSMOOTH, SCROLLVERMIN=-10*CANVASHEIGHT*SCROLLVERSMOOTH,SCROLLVERMAX=10*CANVASHEIGHT*SCROLLVERSMOOTH;
	static final int NODEFRAMEWIDTH=350, NODEFRAMEHEIGHT=250;
	static final int CONFIGFRAMEWIDTH=250, CONFIGFRAMEHEIGHT=150,MAXSIZEZOOM=10000;
	static final int TEXTFIELDLENGTH=20;
	static final int WIDTHMAX=300;
	static final int STEPREFRESH=10000; // times refresh calls before repaint canvas
	static int refresh=0; // counter for refreshing canvas

	// display variables
	boolean recomputeDisplay=true; // true if we need to refresh display data
	boolean compactEdgeDisplay=false;
	double alpha=0,beta=0,gamma=0;
	double cosA=Math.cos(alpha),cosB=Math.cos(beta),sinA=Math.sin(alpha),sinB=Math.sin(beta);
	double cosG=Math.cos(gamma),sinG=Math.sin(gamma);
	boolean is2D=false;
	
	// data variables	
    PropagationRequest pr;
	PropagationTree propagationTree; 
	DataStructRequest datastructrequest; 
	Hashtable activateTextField=new Hashtable();
	int dataType,solverType,port;
	String hostFile;
	
	////////////////////////
	// Declare components //
	////////////////////////
	
	JFrame mainFrame;
	JDesktopPane desk=new JDesktopPane();
	
	// Graphic Frame
	JInternalFrame graphicframe; // frame where is the tree.
	CT_Canvas graphic_location=new CT_Canvas();
	CT_Scrollbar scrollBarHor=new CT_Scrollbar(Scrollbar.HORIZONTAL,SCROLLHORVAL,1,SCROLLHORMIN,SCROLLHORMAX,CANVASWIDTH,10);
	CT_Scrollbar scrollBarVer=new CT_Scrollbar(Scrollbar.VERTICAL,SCROLLVERVAL,1,SCROLLVERMIN,SCROLLVERMAX,CANVASHEIGHT,10);
	CTMenuBar menuBarGraphic;
	
	// Node Informations
	JInternalFrame nodeFrame; // frame with node informations.
	//
	JTextField typenodeText=new JTextField("",TEXTFIELDLENGTH); // Comment Field
	JTextField constraintText=new JTextField("",TEXTFIELDLENGTH);
	JTextField nbTotalText=new JTextField("",TEXTFIELDLENGTH);
	JTextField christmasBowlText=new JTextField("",TEXTFIELDLENGTH);
	JTextField activateText=new JTextField("",TEXTFIELDLENGTH);
	JTextArea detailNodeText= new JTextArea(10,20);

	// Graphic configuration
	JInternalFrame configFrame; // frame for graphic configuration
	JScrollBar zoomXScrollBar=new JScrollBar(JScrollBar.HORIZONTAL,100,1,1,MAXSIZEZOOM);
	JScrollBar zoomYScrollBar=new JScrollBar(JScrollBar.HORIZONTAL,100,1,1,MAXSIZEZOOM);
	JScrollBar widthScrollBar=new JScrollBar(JScrollBar.HORIZONTAL,50,1,0,WIDTHMAX);
	
	// rotation configuration
	JInternalFrame rotationFrame; // frame for rotation
	JScrollBar rotAScrollBar=new JScrollBar(JScrollBar.HORIZONTAL,0,1,-315,315);
	JScrollBar rotBScrollBar=new JScrollBar(JScrollBar.HORIZONTAL,0,1,-315,315);
	JScrollBar rotGScrollBar=new JScrollBar(JScrollBar.HORIZONTAL,0,1,-315,315);
	
	// propagation frame
	JInternalFrame propagationFrame; // frame for select the propagation representation
	Vector reduceFilterCheckBox=new Vector();
	Vector awakeFilterCheckBox=new Vector();
	JCheckBox allreduceCB,allawakeCB;
	
	public CTViewer(PropagationTree pt,int st, int dt, String hf, int p, PropagationRequest pr) {
        this.pr=pr;
		JFrame.setDefaultLookAndFeelDecorated(true);
		JPanel jpanel;
		JLabel jlabel;
		// Type of solver and data
		solverType=st;
		dataType=dt;
		port=p;		
		if(dataType==UISolverSelection.SOCKET) 
			hostFile=hf;
		else {
			String[] af=hf.split("/");
			hostFile=af[af.length-1];
		}
		propagationTree=pt;
		is2D=false;

		
		/////////////////////
		// Desktop config
		mainFrame=new JFrame();
		mainFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		mainFrame.setSize(1100,800);
		mainFrame.setVisible(true);
		mainFrame.setContentPane(desk);
		// Title
		if(solverType==SolverType.GPROLOG && dataType==UISolverSelection.SOCKET) mainFrame.setTitle("Propagation Tree : Codeine by socket. Host="+hostFile+"; Port="+port);
		if(solverType==SolverType.GPROLOG && dataType==UISolverSelection.FILE) mainFrame.setTitle("Propagation Tree : Codeine readed in the file="+hostFile);
		if(solverType==SolverType.CLPGUI && dataType==UISolverSelection.SOCKET) mainFrame.setTitle("Propagation Tree : CLPGUI by socket. Host="+hostFile+"; Port="+port);
		if(solverType==SolverType.CLPGUI && dataType==UISolverSelection.FILE) mainFrame.setTitle("Propagation Tree : CLPGUI readed in the file="+hostFile);
		// Menu
		menuBarGraphic=new CTMenuBar();
		mainFrame.setJMenuBar(menuBarGraphic);
		

		this.initConfigFrame();
		this.initRotationFrame();
		this.initNodeFrame();
		this.initPropagationFrame();

		/////////////////////
		// init graphic frame 
		graphicframe = new JInternalFrame("Propagation Tree",true,false,true,true);
		
		// Content Pane
		graphic_location.start();
		graphicframe.getContentPane().setLayout(new BorderLayout());
		graphicframe.getContentPane().add(graphic_location,BorderLayout.CENTER);
		graphicframe.getContentPane().add(scrollBarVer,BorderLayout.EAST);
		graphicframe.getContentPane().add(scrollBarHor,BorderLayout.SOUTH);
		//
		graphicframe.setLocation(NODEFRAMEWIDTH,0);
		graphicframe.setSize(CANVASWIDTH,CANVASHEIGHT);
		graphicframe.setVisible(true);		
		graphicframe.getContentPane().setVisible(true);		
		desk.add(graphicframe);
		desk.setLayer(graphicframe,0);
	}
	
	// Propgation frame
	private void initPropagationFrame(){
        
		JPanel jpanel;
		JLabel jlabel;
		
		//////////////////////////////////////
		// init propagation frame		
		propagationFrame=new JInternalFrame("Propagation",true,true,false,true);
		propagationFrame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		propagationFrame.getContentPane().setLayout(new BoxLayout(propagationFrame.getContentPane(),BoxLayout.Y_AXIS));
		
		// Reduce
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		jpanel.add(new JLabel("Reduce evnts:"));
		propagationFrame.getContentPane().add(jpanel);
		
		//////////
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		allreduceCB=new JCheckBox("All Reduce (no filter)",true);
		allreduceCB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				if(!reduceFilterCheckBox.isEmpty()) {
				Enumeration enu=reduceFilterCheckBox.elements();
				while(enu.hasMoreElements()) {
					JCheckBox jcbcurr=(JCheckBox)enu.nextElement();
					jcbcurr.setEnabled(!((JCheckBox)(e.getSource())).isSelected());
					}
				}
				graphic_location.computeRepInfo();
				graphic_location.repaint();
			}
			});
		jpanel.add(allreduceCB);
		propagationFrame.getContentPane().add(jpanel);
		/////////
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		
		if(pr.isFilter("reduce")){
			jpanel.add(new JLabel("Filters:"));
			propagationFrame.getContentPane().add(jpanel);
			Vector constraintFilters=pr.getConstraintFilters("reduce");
			Enumeration e=constraintFilters.elements();
			while(e.hasMoreElements()){
				String filter=(String)e.nextElement();
				JCheckBox jcb=new JCheckBox(filter,false);
				jcb.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e){
						if(((JCheckBox)e.getSource()).isSelected())
							allreduceCB.setSelected(false);
						graphic_location.computeRepInfo();
						graphic_location.repaint();
						}
				});
				jcb.setEnabled(false);
				reduceFilterCheckBox.add(jcb);
				jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
				jpanel.add(jcb);
				propagationFrame.getContentPane().add(jpanel);
			}

		}
		propagationFrame.getContentPane().add(this.createSeparator(SwingConstants.HORIZONTAL,CONFIGFRAMEWIDTH,10));
		/// Awake
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		jpanel.add(new JLabel("Awake events:"));
		propagationFrame.getContentPane().add(jpanel);
		
		//////////
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		allawakeCB=new JCheckBox("All Awake (no filter)",true);
		allawakeCB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				if(!awakeFilterCheckBox.isEmpty()) {
					Enumeration enu=awakeFilterCheckBox.elements();
					while(enu.hasMoreElements()) {
						JCheckBox jcbcurr=(JCheckBox)enu.nextElement();
						jcbcurr.setEnabled(!((JCheckBox)(e.getSource())).isSelected());
					}
				}
				graphic_location.computeRepInfo();
				graphic_location.repaint();
			}
		});
		jpanel.add(allawakeCB);
		propagationFrame.getContentPane().add(jpanel);
		/////////
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		
		if(pr.isFilter("awake")){
			jpanel.add(new JLabel("Filters:"));
			propagationFrame.getContentPane().add(jpanel);
			Vector constraintFilters=pr.getConstraintFilters("awake");
			Enumeration e=constraintFilters.elements();
			while(e.hasMoreElements()){
				String filter=(String)e.nextElement();
				JCheckBox jcb=new JCheckBox(filter,false);
				jcb.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e){
						if(((JCheckBox)e.getSource()).isSelected())
							allawakeCB.setSelected(false);
						graphic_location.computeRepInfo();
						graphic_location.repaint();
						}
				});
				jcb.setEnabled(false);
				awakeFilterCheckBox.add(jcb);
				jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
				jpanel.add(jcb);
				propagationFrame.getContentPane().add(jpanel);
			}

		}

		////////////////////////////////
		propagationFrame.setSize(CONFIGFRAMEWIDTH,CONFIGFRAMEHEIGHT+20*reduceFilterCheckBox.size()+20*awakeFilterCheckBox.size());
		propagationFrame.setVisible(is2D==false);
		propagationFrame.getContentPane().setVisible(true);
		propagationFrame.setLocation(0,2*CONFIGFRAMEHEIGHT+NODEFRAMEHEIGHT);
		desk.add(propagationFrame);
		desk.setLayer(propagationFrame,1);
	}
	
	// Init rotation frame
	private void initRotationFrame(){
		JPanel jpanel;
		JLabel jlabel;

		//////////////////////////////////////
		// init rotation frame
		rotationFrame=new JInternalFrame("Rotation",true,true,false,true);
		rotationFrame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		rotationFrame.getContentPane().setLayout(new BoxLayout(rotationFrame.getContentPane(),BoxLayout.Y_AXIS));

		//
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		jlabel=new JLabel("alpha:");
		jlabel.setMinimumSize(new Dimension(80,20));
		jlabel.setPreferredSize(new Dimension(80,20));
		jlabel.setMaximumSize(new Dimension(80,20));
		jpanel.add(jlabel);
		rotAScrollBar.addAdjustmentListener(new RotAdjustmentListener());
		jpanel.add(rotAScrollBar);
		rotationFrame.getContentPane().add(jpanel);
		//
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		jlabel=new JLabel("beta:");
		jlabel.setMinimumSize(new Dimension(80,20));
		jlabel.setPreferredSize(new Dimension(80,20));
		jlabel.setMaximumSize(new Dimension(80,20));
		jpanel.add(jlabel);
		rotBScrollBar.addAdjustmentListener(new RotAdjustmentListener());
		jpanel.add(rotBScrollBar);
		rotationFrame.getContentPane().add(jpanel);
		//
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		jlabel=new JLabel("gamma:");
		jlabel.setMinimumSize(new Dimension(80,20));
		jlabel.setPreferredSize(new Dimension(80,20));
		jlabel.setMaximumSize(new Dimension(80,20));
		jpanel.add(jlabel);
		rotGScrollBar.addAdjustmentListener(new RotAdjustmentListener());
		jpanel.add(rotGScrollBar);
		rotationFrame.getContentPane().add(jpanel);

		//
		rotationFrame.setSize(CONFIGFRAMEWIDTH,CONFIGFRAMEHEIGHT);
		rotationFrame.setVisible(is2D==false);
		rotationFrame.getContentPane().setVisible(true);
	
		rotationFrame.setLocation(0,CONFIGFRAMEHEIGHT);
		desk.add(rotationFrame);
		desk.setLayer(rotationFrame,1);
	}
	// rotation enable or not
	private void rotationFrameSetEnabled(boolean b) {
		rotAScrollBar.setEnabled(b);
		rotBScrollBar.setEnabled(b);
		rotGScrollBar.setEnabled(b);
	}

	// Init configuration frame
	private void initConfigFrame(){
		JPanel jpanel;
		JLabel jlabel;

		//////////////////////////////////////
		// init graphic configuration frame
		configFrame=new JInternalFrame("Zoom",true,true,false,true);
		configFrame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		configFrame.getContentPane().setLayout(new BoxLayout(configFrame.getContentPane(),BoxLayout.Y_AXIS));

		//
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		jlabel=new JLabel("Width tree:");
		jlabel.setMinimumSize(new Dimension(80,20));
		jlabel.setPreferredSize(new Dimension(80,20));
		jlabel.setMaximumSize(new Dimension(80,20));
		jpanel.add(jlabel);
		zoomXScrollBar.addAdjustmentListener(new ZoomAdjustmentListener());
		jpanel.add(zoomXScrollBar);
		configFrame.getContentPane().add(jpanel);
		//
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		jlabel=new JLabel("Height tree:");
		jlabel.setMinimumSize(new Dimension(80,20));
		jlabel.setPreferredSize(new Dimension(80,20));
		jlabel.setMaximumSize(new Dimension(80,20));
		jpanel.add(jlabel);
		zoomYScrollBar.addAdjustmentListener(new ZoomAdjustmentListener());
		jpanel.add(zoomYScrollBar);
		configFrame.getContentPane().add(jpanel);
		//
		jpanel=createJPanel(CONFIGFRAMEWIDTH-10,50,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		jlabel=new JLabel("Propagation:");
		jlabel.setMinimumSize(new Dimension(80,20));
		jlabel.setPreferredSize(new Dimension(80,20));
		jlabel.setMaximumSize(new Dimension(80,20));
		jpanel.add(jlabel);
		widthScrollBar.addAdjustmentListener(new AdjustmentListener(){
			public void adjustmentValueChanged(AdjustmentEvent e){
				graphic_location.repaint();
			}
		});
		jpanel.add(widthScrollBar);
		widthScrollBar.setEnabled(propagationTree.hasPropagation());
		configFrame.getContentPane().add(jpanel);

		//
		configFrame.setSize(CONFIGFRAMEWIDTH,CONFIGFRAMEHEIGHT);
		configFrame.setVisible(true);
		configFrame.getContentPane().setVisible(true);
	
		configFrame.setLocation(0,0);
		desk.add(configFrame);
		desk.setLayer(configFrame,1);
	}
	
	// Init node frame
	private void initNodeFrame(){
		//////////////////
		// init node frame
		nodeFrame=new JInternalFrame("Node",true,true,false,true);
		nodeFrame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		nodeFrame.setContentPane(new JPanel());
		nodeFrame.getContentPane().setLayout(new BoxLayout(nodeFrame.getContentPane(),BoxLayout.Y_AXIS));
		//
		JPanel jpanel;
		JLabel jlabel=new JLabel("Type of node:",SwingConstants.LEFT);
		jlabel.setMinimumSize(new Dimension(180,30));
		jlabel.setPreferredSize(new Dimension(180,30));
		jlabel.setMaximumSize(new Dimension(180,30));
		jpanel=createJPanel();
		jpanel.add(jlabel);
		jpanel.add(typenodeText);
		nodeFrame.getContentPane().add(jpanel);		
		//
		jlabel=new JLabel("Constraint:",SwingConstants.LEFT);
		jlabel.setMinimumSize(new Dimension(180,30));
		jlabel.setPreferredSize(new Dimension(180,30));
		jlabel.setMaximumSize(new Dimension(180,30));
		jpanel=createJPanel();
		jpanel.add(jlabel);
		jpanel.add(constraintText);
		nodeFrame.getContentPane().add(jpanel);
		//
		jlabel=new JLabel("Total of propagation events:",SwingConstants.LEFT);
		jlabel.setMinimumSize(new Dimension(180,30));
		jlabel.setPreferredSize(new Dimension(180,30));
		jlabel.setMaximumSize(new Dimension(180,30));
		jpanel=createJPanel();
		jpanel.add(jlabel);
		jpanel.add(nbTotalText);
		nodeFrame.getContentPane().add(jpanel);
		//
		jlabel=new JLabel("Details of propagation:",SwingConstants.LEFT);
		jlabel.setMinimumSize(new Dimension(180,30));
		jlabel.setPreferredSize(new Dimension(180,30));
		jlabel.setMaximumSize(new Dimension(180,30));
		jpanel=createJPanel(NODEFRAMEWIDTH-10,300,Component.LEFT_ALIGNMENT,Component.TOP_ALIGNMENT);
		detailNodeText.setBorder(BorderFactory.createLineBorder(Color.black));
		detailNodeText.setMinimumSize(new Dimension(100,100));
		detailNodeText.setPreferredSize(new Dimension(300,500));
		detailNodeText.setMaximumSize(new Dimension(620,800));
		jpanel.add(jlabel);
		jpanel.add(detailNodeText);
		nodeFrame.getContentPane().add(jpanel);
		//
		nodeFrame.setSize(NODEFRAMEWIDTH,NODEFRAMEHEIGHT);
		nodeFrame.setVisible(true);		
		nodeFrame.getContentPane().setVisible(true);	

		nodeFrame.setLocation(0,2*CONFIGFRAMEHEIGHT);
		desk.add(nodeFrame);
		desk.setLayer(nodeFrame,1);
	
	}
	// Method for generated appropriate JPanel
	private JPanel createJPanel() {
		JPanel jpanel=new JPanel();

		jpanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		jpanel.setAlignmentY(Component.TOP_ALIGNMENT);
		jpanel.setSize(new Dimension(NODEFRAMEWIDTH-10,30));
		jpanel.setMinimumSize(new Dimension(NODEFRAMEWIDTH-200,30));
		jpanel.setPreferredSize(new Dimension(NODEFRAMEWIDTH-10,30));
		jpanel.setMaximumSize(new Dimension(800,30));
		jpanel.setLayout(new BoxLayout(jpanel,BoxLayout.X_AXIS));
		return jpanel;
	}
	private JPanel createJPanel(int w, int h, float xalign, float yalign) {
		JPanel jpanel=new JPanel();

		jpanel.setAlignmentX(xalign);
		jpanel.setAlignmentY(yalign);
		jpanel.setSize(new Dimension(w,h));
		jpanel.setMinimumSize(new Dimension(w/3,h/3));
		jpanel.setPreferredSize(new Dimension(w,h));
		jpanel.setMaximumSize(new Dimension(w*3,h));
		jpanel.setLayout(new BoxLayout(jpanel,BoxLayout.X_AXIS));
		return jpanel;
	}

	// Paint 
	public void repaint(){
		graphic_location.repaint();
	}

	// Refresh
	public void refresh(){
		recomputeDisplay=true;
		graphic_location.repaint();
	}

	// Menubar
	public class CTMenuBar extends JMenuBar implements ActionListener{
	
		// Options menu
		JMenu options=new JMenu("Options");
		JMenuItem option_close=new JMenuItem("Close"); 
		JMenuItem option_quit=new JMenuItem("Quit"); 
		// Trees Menu
		JMenu treesMenu= new JMenu("Display");
		JCheckBoxMenuItem riverTree=new JCheckBoxMenuItem("River tree",false);
		JCheckBoxMenuItem christmasTree=new JCheckBoxMenuItem("Christmas tree",true);
		JMenuItem restoreHiddenSubtree=new JMenuItem("Restore hidden subtrees");
		JMenuItem compactTree=new JMenuItem("Compact tree");
		JMenuItem automaticHideTree=new JMenuItem("Automatic hide tree");
		JCheckBoxMenuItem childrenWeight= new JCheckBoxMenuItem("Children weight (2D)",false);
		JCheckBoxMenuItem subtreeWeight= new JCheckBoxMenuItem("Subtree weight (2D)",true);
		JCheckBoxMenuItem tree3D = new JCheckBoxMenuItem("Tree 3D",false);
		JMenuItem center=new JMenuItem("Center");
		JMenuItem reset=new JMenuItem("Reset");
		// Window Menu
		JMenu menuWindow=new JMenu("Window");
		JMenuItem nodeInfo=new JMenuItem("Node information");
		JMenuItem graphConfig=new JMenuItem("Zoom");
		JMenuItem rotationConfig=new JMenuItem("Rotation");
		JMenuItem propagationFilter=new JMenuItem("Propagation");
		
		public CTMenuBar(){
			//
			options.add(option_close);
			options.add(option_quit);
			
			option_close.setActionCommand("Close");
			option_quit.setActionCommand("Quit");
			option_close.addActionListener(this);		
			option_quit.addActionListener(this);		
			//
			childrenWeight.setActionCommand("childrenWeight");
			childrenWeight.addActionListener(this);
			treesMenu.add(childrenWeight);
			subtreeWeight.setActionCommand("subtreeWeight");
			subtreeWeight.addActionListener(this);
			treesMenu.add(subtreeWeight);
			tree3D.setActionCommand("tree3D");
			tree3D.addActionListener(this);
			treesMenu.add(tree3D);
			treesMenu.addSeparator();
			riverTree.addActionListener(this);
			riverTree.setEnabled(propagationTree.hasPropagation());
			treesMenu.add(riverTree);
			christmasTree.addActionListener(this);
			christmasTree.setEnabled(propagationTree.hasPropagation());
			treesMenu.add(christmasTree);
			treesMenu.addSeparator();
			treesMenu.add(restoreHiddenSubtree);
			restoreHiddenSubtree.setActionCommand("Restore hidden subtrees");
			treesMenu.add(automaticHideTree);
			treesMenu.add(compactTree);
			restoreHiddenSubtree.addActionListener(this);
			automaticHideTree.addActionListener(this);
			compactTree.addActionListener(this);
			treesMenu.addSeparator();
			treesMenu.add(center);
			center.setActionCommand("Center");
			center.addActionListener(this);
			treesMenu.add(reset);
			reset.addActionListener(this);
			//
			menuWindow.add(nodeInfo);
			menuWindow.add(graphConfig);
			menuWindow.add(rotationConfig);
			menuWindow.add(propagationFilter);
			nodeInfo.setActionCommand("Node Info");
			nodeInfo.addActionListener(this);
			graphConfig.setActionCommand("Graph Config");
			graphConfig.addActionListener(this);
			rotationConfig.setActionCommand("Rotation");
			rotationConfig.addActionListener(this);
			propagationFilter.setActionCommand("Propagation");
			propagationFilter.addActionListener(this);

			

			//
			add(options);
			add(treesMenu);
			add(menuWindow);
		}
		
		
		// Action events
		public void actionPerformed(ActionEvent e) { 
			if (e.getActionCommand()=="Quit") { System.exit(0);}
			else if (e.getActionCommand()=="Close") {
				mainFrame.dispose();
			}
			else if (e.getActionCommand()=="Center") {
				graphic_location.center();}
			else if(e.getActionCommand()=="subtreeWeight") {
				subtreeWeight.setSelected(true);
				childrenWeight.setSelected(false);
				tree3D.setSelected(false);
				recomputeDisplay=true;
				is2D=true;
				rotationFrameSetEnabled(false);
				graphic_location.setTreeFeature(graphic_location.SUBTREEWEIGHT);
				graphic_location.repaint();
			}
			else if(e.getActionCommand()=="childrenWeight") {
				subtreeWeight.setSelected(false);
				childrenWeight.setSelected(true);
				tree3D.setSelected(false);
				recomputeDisplay=true;
				is2D=true;
				rotationFrameSetEnabled(false);
				graphic_location.setTreeFeature(graphic_location.CHILDRENWEIGHT);
				graphic_location.repaint();
			}
			else if(e.getActionCommand()=="tree3D") {
				subtreeWeight.setSelected(false);
				childrenWeight.setSelected(false);
				tree3D.setSelected(true);
				recomputeDisplay=true;
				is2D=false;
				rotationFrameSetEnabled(true);
				graphic_location.setTreeFeature(graphic_location.TREE3D);
				graphic_location.repaint();
			}
			else if (e.getActionCommand()=="Christmas tree") {
				christmasTree.setSelected(true);
				riverTree.setSelected(false);
				recomputeDisplay=true;
				graphic_location.setPropRepresentation(graphic_location.CHRISTMASTREE);
				graphic_location.repaint();
			}
			else if (e.getActionCommand()=="River tree") {
				christmasTree.setSelected(false);
				riverTree.setSelected(true);
				recomputeDisplay=true;
				graphic_location.computeRepInfo();
				graphic_location.setPropRepresentation(graphic_location.RIVERTREE);
				graphic_location.repaint();
			}
			else if (e.getActionCommand()=="Restore hidden subtrees") {
				propagationTree.showAll();
				propagationTree.compute_mass();

				recomputeDisplay=true;
				graphic_location.repaint();
			}
			else if (e.getActionCommand()=="Automatic hide tree") {
				if(propagationTree.isAutomaticHidden()) propagationTree.setAutomaticHide(false);
				else propagationTree.setAutomaticHide(true);
				recomputeDisplay=true;
				graphic_location.repaint();
			}
			else if (e.getActionCommand()=="Compact tree") {
				if(compactEdgeDisplay) {
					compactEdgeDisplay=false;
					graphic_location.repaint();
				}
				else {
					compactEdgeDisplay=true;
					PropagationTree.computeCompactEdge(propagationTree);
					graphic_location.repaint();
				}
			}
			else if (e.getActionCommand()=="Node Info") {
				if(nodeFrame.isDisplayable()) nodeFrame.toFront();
				else {
					desk.add(nodeFrame);
					nodeFrame.show();
					nodeFrame.toFront();
					
				}
			}
			else if (e.getActionCommand()=="Graph Config") {
				if(configFrame.isDisplayable()) configFrame.toFront();
				else {
					desk.add(configFrame);
					configFrame.show();
					configFrame.toFront();
				}
			}
			else if (e.getActionCommand()=="Rotation" && (is2D==false)) {
				if(rotationFrame.isDisplayable()) rotationFrame.toFront();
				else {
					desk.add(rotationFrame);
					rotationFrame.show();
					rotationFrame.toFront();
				}
			}
			else if (e.getActionCommand()=="Propagation") {
				if(propagationFrame.isDisplayable()) propagationFrame.toFront();
				else {
					desk.add(propagationFrame);
					propagationFrame.show();
					propagationFrame.toFront();
				}
			}
			else if (e.getActionCommand()=="Reset") {
				zoomXScrollBar.setValue(100);
				zoomYScrollBar.setValue(100);
				widthScrollBar.setValue(100);
				rotAScrollBar.setValue(0);
				rotBScrollBar.setValue(0);
				rotGScrollBar.setValue(0);
				graphic_location.init();
				recomputeDisplay=true;
				repaint();
			}
		}
		
	}

		// CT_Canvas : Canvas class for Christmas Tree
	public  class CT_Canvas extends JPanel implements MouseListener, MouseMotionListener{
		
		// Constants
		static final int ZROOTINIT=-SCROLLVERVAL/SCROLLVERSMOOTH;
		static final int DEEPLENGTH=20; // standard length of tree's branch
		static final int ZOOMCST=8; // constant for zoom coefficient
		static final int BOWLWIDTH=10;
		static final double RIVERWIDTH=10;
		static final int CHILDRENWEIGHT=1, SUBTREEWEIGHT=2, TREE3D=3;// tree feature
		static final int CHRISTMASTREE=1, RIVERTREE=2;// propagation representation

		//
		double mouseZ=0,mouseY=0; // Coord. of the mouse in the canvas
		int depthMax=0,depthMaxCurr=0; // Depth maximal of the tree
		int canvasWidth=CANVASWIDTH; // width of the canvas display
		int canvasHeight=CANVASHEIGHT; // height of the canvas display
		PropagationTree propagationTreeHighlighted=null; // Search tree highlight with the mouse
		boolean propagationTreeHighlightBool=true;
		int bowlWidth=BOWLWIDTH;
		double riverWidth=RIVERWIDTH;
		int treeFeature=CHILDRENWEIGHT;
		int propRepresentation=CHRISTMASTREE;
		
		/// Zoom Variables
		int treeWidthZoom;
		double yZoom0,zZoom0; // Coord. of init point for zoom
		boolean zoom=false; // doing zoom?
		double zoomYCoeff=1,zoomZCoeff=1; // coefficients of zoom 
		double yroot0,zroot0; // root in begining of zoom (mouse 2 pressed)
		double deltaY,deltaZ; // dist. between zoom center and root
		int deepLength,deepLengthZoom; // actually length of tree's branch
		int treeWidth=CANVASWIDTH; // real width of the tree (may be larger than the display)

		double yroot,zroot; // Coord. of the tree's root

		// double buffer
		//Image canvasDB;

		// Paint
		public void paint(Graphics g){			
			super.paint(g);

			if(propagationTree!=null) {
				yroot=this.getWidth()-scrollBarHor.getValue()/SCROLLHORSMOOTH;
				zroot=-scrollBarVer.getValue()/SCROLLVERSMOOTH;
				if((treeFeature==SUBTREEWEIGHT)&& recomputeDisplay) {
					propagationTree.recomputeDisplayMass(treeWidthZoom,deepLengthZoom);
					this.computeRepInfo();
					recomputeDisplay=false;
				}
				else if((treeFeature==CHILDRENWEIGHT)&& recomputeDisplay) {
					propagationTree.recomputeDisplayStandard(treeWidthZoom,deepLengthZoom,is2D);
					this.computeRepInfo();
					recomputeDisplay=false;
				}
				else if(treeFeature==TREE3D && recomputeDisplay){
					propagationTree.recomputeDisplayStandard(treeWidthZoom,deepLengthZoom,is2D);
					this.computeRepInfo();
					recomputeDisplay=false;
				}
				if(is2D)
					drawTree(g,propagationTree,yroot,zroot);
				else 
					drawTree3D(g,propagationTree,yroot,zroot,0);
			}
		}

		// Starts
		public void start(){
			this.setSize(canvasWidth,canvasHeight);
			this.setBackground(Color.white);
			this.setOpaque(true);
			this.addMouseListener(this);
			this.addMouseMotionListener(this);
			this.setVisible(true);
			this.setDoubleBuffered(true);
			this.init();
		}
		
		// Initialize root
		public void init(){			
			canvasWidth=this.getWidth();
			canvasHeight=this.getHeight();
			treeWidth=canvasWidth;
			yroot=-SCROLLHORVAL/SCROLLHORSMOOTH+canvasWidth/2;
			zroot=ZROOTINIT;
			scrollBarHor.setValue(SCROLLHORSMOOTH*this.getWidth()/2);
			scrollBarVer.setValue(SCROLLVERVAL);
			zoomYCoeff=1;
			zoomZCoeff=1;
			deepLength=DEEPLENGTH;
			deepLengthZoom=DEEPLENGTH;
			treeWidthZoom=CANVASWIDTH;
		}

		// Center the tree in the panel
		public void center(){			
			scrollBarHor.setValue(SCROLLHORSMOOTH*this.getWidth()/2);
			scrollBarVer.setValue(SCROLLVERVAL);
			this.repaint();
		}

		// reinit deepLength and treeWidth
		public void reinit(){
			treeWidthZoom=Math.max((int)((treeWidth*zoomXScrollBar.getValue())/100),100);
			deepLengthZoom=Math.max((int)((DEEPLENGTH*zoomYScrollBar.getValue())/100),2);
			depthMax=Math.max((int)((depthMax*zoomYScrollBar.getValue())/100),2);
		}
		// Set root
		public void setRoot(double y, double z){
			yroot=y;
			zroot=z;
		}

		// Set tree feature
		public void setTreeFeature(int nf){ 
			treeFeature=nf;
			recomputeDisplay=true;
		}

		// Set tree feature
		public void setPropRepresentation(int pr){ propRepresentation=pr; }
		
		// Set bowl width
		public void setBowlWidth(int bw){ bowlWidth=bw; }

		// Set bowl width
		public void setRiverWidth(int rw){ riverWidth=rw; }
		
		// Resize
		public void resize(){
			canvasWidth=this.getWidth();
			canvasHeight=this.getHeight();
		}


		public void drawTree(Graphics g, PropagationTree t, double y, double z){
			double y0=y;
			double z0=z;
			Vector c;
			Enumeration e;
			
			// Display comments if the mouse is close to (y0,z0)
			if(propagationTreeHighlightBool&&((z-mouseZ)*(z-mouseZ)+(y-mouseY)*(y-mouseY)<6)) {				
				propagationTreeHighlightBool=false;
				propagationTreeHighlighted=t;
				detailNodeDisplay(t);
			}


				
			drawNode(y0,z0,g,t.getRepDisplayNode(),t.getLabel());
			if(t.children()) {
				c=t.getChildren();
				e=c.elements();
			}
			else {
				return ; }

			if(t.isHiddenTree()) {
				int widthHiddenTree=Math.min(Math.max(2,t.getWidthDisplay()-2),20);
				drawHiddenSubtree(y0,z0,widthHiddenTree,g,t.containsSolution());
				return ;
			}
			
			if(compactEdgeDisplay&&t.isCompactEdge()){
				drawCompactedEdge(y0,z0,2*deepLengthZoom,g);
				drawTree(g,t.down(t.getNbAloneChildren()),y,z+2*deepLengthZoom);
				return;
			}
			
			// Compute tree depth
			depthMaxCurr=depthMaxCurr+deepLengthZoom;
			if(depthMax<depthMaxCurr) {
				depthMax=depthMaxCurr;
				graphic_location.reinit();
			}
			z=z0+deepLengthZoom;
			while(e.hasMoreElements()) {
				PropagationTree subtree=(PropagationTree)e.nextElement();
				y=yroot+subtree.getRelativeCoordinate().getX();
				drawEdge(y0,z0,y,z,g,subtree.getStrahler(),propagationTree.getStrahler());
				drawTree(g,subtree,y,z);
			}
			depthMaxCurr=depthMaxCurr-deepLengthZoom;
			//drawNode(y0,z0,g,t.getNbTotalPropagation(),t.getLabel());
			return ;
			
		} 

		// Draw tree 3D
		public void drawTree3D(Graphics g, PropagationTree t, double y, double z,double deep){
			double y0=y;
			double z0=z;
			Vector c;
			Enumeration e;
			
			drawNode(y0,z0,g,t.getRepDisplayNode(),t.getLabel());
						
			// Display comments if the mouse is close to (y0,z0)
			if(propagationTreeHighlightBool&&((z-mouseZ)*(z-mouseZ)+(y-mouseY)*(y-mouseY)<6)) {				
				propagationTreeHighlightBool=false;
				propagationTreeHighlighted=t;
				detailNodeDisplay(t);
				
			}

			if(t.children()) {
				c=t.getChildren();
				e=c.elements();
			}
			else {
				return ; }

			if(t.isHiddenTree()) {
				int widthHiddenTree=Math.min(Math.max(2,t.getWidthDisplay()-2),20);
				drawHiddenSubtree3D((double)t.getRelativeCoordinate3D().getX(),(double)t.getRelativeCoordinate3D().getY(),deep,(double)widthHiddenTree,g,t.containsSolution());
				return;
			}

			if(compactEdgeDisplay&&t.isCompactEdge()){
				drawCompactedEdge3D((double)t.getRelativeCoordinate3D().getX(),(double)t.getRelativeCoordinate3D().getY(),deep,2*deepLengthZoom,g);
				double ynew=yroot+i((double)t.getRelativeCoordinate3D().getX(),(double)t.getRelativeCoordinate3D().getY(),deep+2*deepLengthZoom);
				double znew=zroot+j((double)t.getRelativeCoordinate3D().getX(),(double)t.getRelativeCoordinate3D().getY(),deep+2*deepLengthZoom);
				drawTree3D(g,t.down(t.getNbAloneChildren()),ynew,znew,deep+2*deepLengthZoom);
				return;
			}

			// Compute tree depth
			depthMaxCurr=depthMaxCurr+deepLengthZoom;
			if(depthMax<depthMaxCurr) {
				depthMax=depthMaxCurr;
				graphic_location.reinit();
			}
			
			deep=deep+deepLengthZoom;
			while(e.hasMoreElements()) {
				PropagationTree subtree=(PropagationTree)e.nextElement();
				y=yroot+i((double)subtree.getRelativeCoordinate3D().getX(),(double)subtree.getRelativeCoordinate3D().getY(),deep);
				z=zroot+j((double)subtree.getRelativeCoordinate3D().getX(),(double)subtree.getRelativeCoordinate3D().getY(),deep);
				drawEdge(y0,z0,y,z,g,subtree.getStrahler(),propagationTree.getStrahler());
				drawTree3D(g,subtree,y,z,deep);
			}
			depthMaxCurr=depthMaxCurr-deepLengthZoom;
			return ;
		}

		// projection on y.
		private double i(double x, double y, double z) {
			return (cosA*(y*cosB+x*sinB)-z*sinA);			
		}

		// projection on z.
		private double j(double x, double y, double z) {
			double d=(x*cosB-y*sinB);
			
			return cosG*(sinA*(y*cosB+x*sinB)+z*cosA)-d*sinG;		
		}
		
		// Display activate data
		private void detailNodeDisplay(PropagationTree t){
			Enumeration e1;
			detailNodeText.setText("");
			if(t.getDataStructRequest().getRequests()==null) return;
			e1=t.getDataStructRequest().getRequests().keys();
			
			typenodeText.setText(""+t.getLabel());
			nbTotalText.setText(""+t.getRepDisplayNode());
			constraintText.setText(t.getExternalRep());

			while(e1.hasMoreElements()) {				
				Hashtable constraintFilter;
				Enumeration e;	
				String type=(String)e1.nextElement();
				
				detailNodeText.setText(
					detailNodeText.getText()+
					type+": "+t.getDataStructRequest().getTotalEvents(type)+"\n");

				constraintFilter=t.getDataStructRequest().getConstraintFilter(type);
				
				if(constraintFilter!=null){
				e=constraintFilter.keys();
				while(e.hasMoreElements()){
					String s=(String)e.nextElement();
					detailNodeText.setText(
						detailNodeText.getText()+"  "+
						s+": "+t.getDataStructRequest().getConstraintFilterValue(type,s)+"\n");
				}}
				detailNodeText.setText(detailNodeText.getText()+"---------------\n");
			}
		} 		

		// Computes representatif information
		private void computeRepInfo(){
			propagationTree.computeRepInfo(allreduceCB.isSelected(),reduceFilterCheckBox,allawakeCB.isSelected(),awakeFilterCheckBox);
			propagationTree.computeStrahler(propagationTree);
		}

		// Draw hidden subtree
		// k : coefficient for size representation
		private void drawHiddenSubtree(double y, double z, double k, Graphics g, boolean containsSolution){
			int [] ycoord={(int)y,(int)(y-k),(int)(y+k)};
			int [] zcoord={(int)z,(int)(z+deepLengthZoom),(int)(z+deepLengthZoom)};
			if(containsSolution) g.setColor(Color.green);
			else g.setColor(Color.black);
			g.fillPolygon(ycoord,zcoord,3);
			
			return;
		}
		
		private void drawHiddenSubtree3D(double x, double y, double z, double k, Graphics g, boolean containsSolution){
			int [] ycoord={(int)(yroot+i(x,y,z)),(int)(yroot+i(x,y-k,z+deepLengthZoom)),(int)(yroot+i(x,y+k,z+deepLengthZoom))};
			int [] zcoord={(int)(zroot+j(x,y,z)),(int)(zroot+j(x,y-k,z+deepLengthZoom)),(int)(zroot+j(x,y+k,z+deepLengthZoom))};
//			int [] zcoord={(int)z,(int)(z+deepLengthZoom),(int)(z+deepLengthZoom)};
			if(containsSolution) g.setColor(Color.green);
			else g.setColor(Color.black);
			g.fillPolygon(ycoord,zcoord,3);
			
			return;
		}

		
		// Draw compacted edges
		private void drawCompactedEdge(double y, double z, double length, Graphics g){
			int ycurr=(int)y,zcurr=(int)z;
			
			g.setColor(Color.blue);
			while(zcurr<z+length-2){
				g.drawLine(ycurr,zcurr,ycurr,zcurr+2);
				zcurr=zcurr+4;
			}
		}
			
		// Draw compacted edges 3D
		private void drawCompactedEdge3D(double x,double y, double z, double length, Graphics g){
			double zcurr=z;
			int ydisplay1,zdisplay1;
			int ydisplay2,zdisplay2;
			
			g.setColor(Color.blue);
			ydisplay1=(int)(yroot+i(x,y,z));
			zdisplay1=(int)(zroot+j(x,y,z));
			while(zcurr<z+length-2){
				ydisplay2=(int)(yroot+i(x,y,zcurr+2));
				zdisplay2=(int)(zroot+j(x,y,zcurr+2));
				g.drawLine(ydisplay1,zdisplay1,ydisplay2,zdisplay2);
				zcurr=zcurr+4;
				ydisplay1=(int)(yroot+i(x,y,zcurr));
				zdisplay1=(int)(zroot+j(x,y,zcurr));
			}
		}	
		// Draw nodes
		private void drawNode(double y0, double z0, Graphics g, int cbReal, String nodeType){
			if(propRepresentation==CHRISTMASTREE){
				int cb=(int)((double)(widthScrollBar.getValue()*cbReal)/(double)100);
				//real bowl width
				if(cb>0) {
						g.setColor(Color.red);
						g.fillOval((int)(y0-cb/2),(int)(z0-cb/2),cb,cb);
					}
			}
			g.setColor(Color.black);
			g.drawLine((int)y0-2,(int)z0,(int)y0+2,(int)z0);
			g.drawLine((int)y0,(int)z0+2,(int)y0,(int)z0-2);

			if(nodeType=="solution"){
				int [] ypoly={(int)y0-4,(int)y0-4,(int)y0+4,(int)y0+4};
				int [] zpoly={(int)z0-4,(int)z0+4,(int)z0+4,(int)z0-4};
				g.setColor(Color.green);
				g.fillPolygon(ypoly,zpoly,4);
//				g.drawString("S",(int)y0,(int)z0+15);
			}
			else if(nodeType=="failure"){
				int [] ypoly={(int)y0-2,(int)y0-2,(int)y0+2,(int)y0+2};
				int [] zpoly={(int)z0-2,(int)z0+2,(int)z0+2,(int)z0-2};
				g.setColor(Color.black);
				g.fillPolygon(ypoly,zpoly,4 );
//				g.drawString("S",(int)y0,(int)z0+15);
			}

			return;
		}

		// Draw edges
		private void drawEdge(double y0, double z0, double y, double z, Graphics g,double strahler, double strahler0){
			g.setColor(Color.blue);
			if(propRepresentation==CHRISTMASTREE){
				g.drawLine((int)y0,(int)z0,(int)y,(int)z);
			}
			else if(propRepresentation==RIVERTREE) {				
				double riverWidthCurr=(strahler*widthScrollBar.getValue())/(Math.max(strahler0*25,1));
				if(riverWidthCurr>=2) {
					double s=Math.sqrt((y-y0)*(y-y0)+(z-z0)*(z-z0));
					double ly=((z-z0)*riverWidthCurr)/(2*s);
					double lz=(Math.abs(y-y0)*riverWidthCurr)/(2*s);
					int [] ycoord={(int)(y0+ly),(int)(y+ly),(int)(y-ly),(int)(y0-ly),(int)(y0+ly)};
					int [] zcoord=new int [5];


					if(y>y0) {
						zcoord[0]=(int)(z0-lz);
						zcoord[1]=(int)(z-lz);
						zcoord[2]=(int)(z+lz);
						zcoord[3]=(int)(z0+lz);
						zcoord[4]=(int)(z0-lz); } 
					else {
						zcoord[0]=(int)(z0+lz);
						zcoord[1]=(int)(z+lz);
						zcoord[2]=(int)(z-lz);
						zcoord[3]=(int)(z0-lz); 
						zcoord[4]=(int)(z0+lz);}	
					g.fillPolygon(ycoord,zcoord,5);
				}
				else g.drawLine((int)y0,(int)z0,(int)y,(int)z); 
			}
			g.drawLine((int)y0,(int)z0,(int)y,(int)z); 
			return;
		} 
		

		//////////////////
		// Mouse Events //
		//////////////////

		public synchronized void mousePressed(MouseEvent e){			

		}
		public synchronized void mouseReleased(MouseEvent e) {
		}
		public synchronized void mouseEntered(MouseEvent e) {}
		public synchronized void mouseExited(MouseEvent e) {}
		public synchronized void mouseClicked(MouseEvent e) {
			if((e.getModifiers() & InputEvent.BUTTON3_MASK)!=0 && propagationTreeHighlighted!=null) {
				// Sub-CTViewer
				PropagationTree pt=propagationTreeHighlighted.copy();
				CTViewer sub_ct_interface=new CTViewer(pt,solverType,dataType,hostFile,port, pr);
			}
			else if((e.getModifiers() & InputEvent.BUTTON1_MASK)!=0 && propagationTreeHighlighted!=null) {
				// Hides or restores the subtree
				propagationTreeHighlighted.changeDisplayState();
				recomputeDisplay=true;
				this.repaint();
			}
		}
		public synchronized void mouseMoved(MouseEvent e){
			propagationTreeHighlightBool=true;
			propagationTreeHighlighted=null;
			mouseY=e.getX();
			mouseZ=e.getY();
			this.repaint();
		}
		public synchronized void mouseDragged(MouseEvent e){		
		}
	}
	
	/////////////////////
	// ScrollBarEvents //
	/////////////////////
	
	private class ZoomAdjustmentListener implements AdjustmentListener {
		public void adjustmentValueChanged(AdjustmentEvent e) {
			recomputeDisplay=true;
			graphic_location.reinit();
			graphic_location.repaint();
		}
	}
	
	private class RotAdjustmentListener implements AdjustmentListener {
		public void adjustmentValueChanged(AdjustmentEvent e) {
			alpha=((double)rotAScrollBar.getValue())/100;
			beta=((double)rotBScrollBar.getValue())/100;
			gamma=((double)rotGScrollBar.getValue())/100;
			cosA=Math.cos(alpha);
			cosB=Math.cos(beta);
			sinA=Math.sin(alpha);
			sinB=Math.sin(beta);
			cosG=Math.cos(gamma);
			sinG=Math.sin(gamma);
			graphic_location.reinit();
			graphic_location.repaint();
		}
	}
	// Scroll bar class for chrismas tree viewer
	public class CT_Scrollbar extends JScrollBar implements AdjustmentListener{
		
		public CT_Scrollbar(int orientation, int value, int visible, int minimum, int maximum,int width, int height){
		this.setOrientation(orientation);
		this.setVisibleAmount(visible);
		this.setMinimum(minimum);
		this.setMaximum(maximum);
		this.setValue(value);
		this.addAdjustmentListener(this);
		this.setBlockIncrement((maximum-minimum)/50);
		this.setSize(width,height);
		}

		public void adjustmentValueChanged(AdjustmentEvent e){			
			double yrootNew=graphic_location.yroot,zrootNew=graphic_location.zroot;
			if(getOrientation()==Scrollbar.HORIZONTAL)
				yrootNew=-this.getValue()/SCROLLHORSMOOTH+graphic_location.getWidth();
			else 
				zrootNew=-this.getValue()/SCROLLVERSMOOTH;
			graphic_location.setRoot(yrootNew,zrootNew);
			graphic_location.repaint();
		}

		public void reinit(){
//			this.setSize(graphic_location.getWidth(),10);
			this.setBlockIncrement((this.getMaximum()-this.getMinimum())/50); }
	}

	/////////////////////////////////////////////////////////////////
/*	private static void show(){
		CTViewer2 ctviewer=new CTViewer2();
		PropagationTree pt=new PropagationTree("root");
		
		pt.addChild(new PropagationTree("child 1"));
		pt.addChild(new PropagationTree("child 2"));
		pt.down(1).addChild(new PropagationTree("child 11"));
		ctviewer.init(pt);
	}
	
	public static void main(String[] args) {
        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
        	public void run() { show(); } });
	}*/


	///////////////////////////
	// Functions
	///////////////////////////
	public Vector arrayToVector(Object[] o,int length) {
		Vector ret=new Vector(length);
		int i=0;
		while(i<length) {
			ret.add(o[i]);
			i++;
		}
		return ret;
	}

	/**
	 * create separator.
	 */
	private JSeparator createSeparator(int axis,int w,int h){
		JSeparator separator=new JSeparator(axis);
		separator.setPreferredSize(new Dimension(w,h));
		separator.setMaximumSize(new Dimension(w,h));
		separator.setAlignmentX(Component.LEFT_ALIGNMENT);
		return separator;
	}

    public void reshow(){mainFrame.show();}

    public int getType(){return type;}
}

